﻿using System.Linq;
using Outlook.WorkItems.Extension.Addin.Actions;
using Microsoft.Office.Interop.Outlook;
using Microsoft.Office.Core;

namespace Outlook.WorkItems.Extension.Addin
{
    partial class BowieRibbonExtensibility
    {
        public void trackInWorkItemTaskButton_Action(IRibbonControl control, bool isPressed)
        {
            if (isPressed)
            {
                TaskItem item;
                if (!TryGetTaskItemFromInspector(control.Context, out item))
                    return;

                var action = new TrackInWorkItemAction(new OutlookTaskItem(item));
                action.Do();
            }
            else
            {
                //TODO : not implemented
            }
        }

        public void insertReportTaskButton_Action(IRibbonControl control)
        {
            TaskItem item;
            if (!TryGetTaskItemFromInspector(control.Context, out item))
                return;

            var action = new InsertReportAction(item.Attachments);
            action.Do();
        }

        public void downloadAttachmentTaskButton_Action(IRibbonControl control)
        {
            TaskItem item;
            if (!TryGetTaskItemFromInspector(control.Context, out item))
                return;

            var action = new DownloadAttachmentsFromTeamPortalAction(item.Attachments);
            action.Do();

            if (string.IsNullOrEmpty(action.HtmlLink))
                return;

            if (action.HtmlLinks != null && action.HtmlLinks.Count > 0)
            {
                foreach (var htmlLink in action.HtmlLinks)
                {
                    item.Body += string.Format("\r\n{0} : {1}", htmlLink.Item1, htmlLink.Item2);
                }
            }
        }

        public void sendAttachmentTaskButton_Action(IRibbonControl control)
        {
            TaskItem item;
            if (!TryGetTaskItemFromInspector(control.Context, out item))
                return;

            var attachments = item.Attachments.Cast<Attachment>();
            var action = new UploadAttachmentsToTeamPortalAction(attachments);
            action.Do();
        }

        public bool sendAttachmentTaskButton_GetEnabled(IRibbonControl control)
        {
            TaskItem item;
            if (!TryGetTaskItemFromInspector(control.Context, out item))
                return false;

            return item.Attachments != null && item.Attachments.Count > 0;
        }

        public bool trackInWorkItemTaskButton_GetPressed(IRibbonControl control)
        {
            TaskItem item;
            if (!TryGetTaskItemFromInspector(control.Context, out item))
                return false;

            return item.ItemProperties["RelatedWIIds"] != null && item.ItemProperties["RelatedWIIds"].Value != "" && item.ItemProperties["RelatedWIIds"].Value != null;            
        }

        public void relatedWorkItemTaskButton_Action(IRibbonControl control)
        {
            TaskItem item;
            if (!TryGetTaskItemFromInspector(control.Context, out item))
                return;

            var action = new ViewRelatedWorkItemAction(item.ItemProperties);
            action.Do();
        }

        public bool relatedWorkItemTaskButton_GetEnabled(IRibbonControl control)
        {
            TaskItem item;
            if (!TryGetTaskItemFromInspector(control.Context, out item))
                return false;

            return item.ItemProperties["RelatedWIIds"] != null && item.ItemProperties["RelatedWIIds"].Value != "" && item.ItemProperties["RelatedWIIds"].Value != null;            
        }

        public static bool TryGetTaskItemFromInspector(dynamic context, out TaskItem item)
        {
            item = null;

            var inspector = context as Inspector;
            if (inspector == null)
                return false;

            item = inspector.CurrentItem as TaskItem;
            return item != null;
        }
    }
}
